@@ -33,6 +33,9 @@
 #include <string.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib-lowlevel.h>
+#include <sys/types.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
 #include "applet-notifications.h"
 #include "applet-dbus-devices.h"
 #include "applet-dbus.h"
@@ -114,7 +117,7 @@ typedef struct DriverCBData
 /*
  * nma_dbus_device_get_driver_cb
  *
- * Callback from nma_dbus_update_wireless_enabled
+ * Callback from nma_dbus_device_get_driver
  *
  */
 static void nma_dbus_device_get_driver_cb (DBusPendingCall *pcall, void *user_data)
@@ -204,7 +207,9 @@ static void nma_dbus_update_wireless_enabled_cb (DBusPendingCall *pcall, void *u
 {
 	DBusMessage *		reply;
 	NMApplet *	applet = (NMApplet *) user_data;
-	gboolean			wireless_enabled;
+	gboolean wireless_enabled = FALSE;
+	gboolean hw_rf_enabled = TRUE; /* default to true to maintain backward compat */
+	gboolean success = TRUE;
 
 	g_return_if_fail (pcall != NULL);
 	g_return_if_fail (applet != NULL);
@@ -224,9 +229,20 @@ static void nma_dbus_update_wireless_enabled_cb (DBusPendingCall *pcall, void *u
 		goto out;
 	}
 
-	if (dbus_message_get_args (reply, NULL, DBUS_TYPE_BOOLEAN, &wireless_enabled, DBUS_TYPE_INVALID))
+	if (!dbus_message_get_args (reply, NULL,
+	                            DBUS_TYPE_BOOLEAN, &wireless_enabled,
+	                            DBUS_TYPE_BOOLEAN, &hw_rf_enabled,
+	                            DBUS_TYPE_INVALID))
 	{
+		if (!dbus_message_get_args (reply, NULL,
+		                            DBUS_TYPE_BOOLEAN, &wireless_enabled,
+		                            DBUS_TYPE_INVALID))
+			success = FALSE;
+	}
+
+	if (success) {
 		applet->wireless_enabled = wireless_enabled;
+		applet->hw_rf_enabled = hw_rf_enabled;
 		nma_enable_wireless_set_active (applet);
 	}
 
@@ -817,7 +833,11 @@ static void nma_dbus_device_properties_cb (DBusPendingCall *pcall, void *user_da
 				for (item = networks; *item; item++)
 					nma_dbus_device_update_one_network (applet, op, *item, active_network_path);
 			}
+		} else if (type == DEVICE_TYPE_802_3_ETHERNET) {
+			if (active && strlen (active_network_path) > 0)
+				network_device_set_active_wired_network (dev, active_network_path);
 		}
+
 		dbus_free_string_array (networks);
 
 		applet->device_list = g_slist_append (applet->device_list, dev);
@@ -891,24 +911,31 @@ static void nma_dbus_device_activated_cb (DBusPendingCall *pcall, void *user_dat
 
 #ifdef ENABLE_NOTIFY
 	active_device = nma_get_first_active_device (applet->device_list);
-	if (active_device && network_device_is_wireless (active_device))
-	{
-		if (applet->is_adhoc)
-		{
+	if (!active_device)
+		goto out;
+
+	if (network_device_is_wireless (active_device)) {
+		if (applet->is_adhoc) {
 			message = g_strdup_printf (_("You are now connected to the Ad-Hoc wireless network '%s'."), essid);
 			icon = "nm-adhoc";
-		}
-		else
-		{
+		} else {
 			message = g_strdup_printf (_("You are now connected to the wireless network '%s'."), essid);
 			icon = "nm-device-wireless";
 		}
+	} else {
+		const char *str_addr = network_device_get_ip4_address (active_device);
 		
-	}
-	else
-	{
-		message = g_strdup (_("You are now connected to the wired network."));
-		icon = "nm-device-wired";
+		if (str_addr && !strncmp (str_addr, LL_ADDR_PREFIX, strlen (LL_ADDR_PREFIX))) {
+			message = g_strdup (_("You are now connected to the wired network with a self-assigned address."));
+			icon = "nm-device-wired-autoip";
+		} else {
+			if (essid)
+				message = g_strdup_printf (_("You are now connected to the wired network '%s'."), essid);
+			else
+				message = g_strdup (_("You are now connected to the wired network."));
+
+			icon = "nm-device-wired";
+		}
 	}
 
 	nm_info ("%s", message);
@@ -1267,28 +1294,62 @@ void nma_dbus_set_device (DBusConnection *connection, NetworkDevice *dev, const
 	{
 		const char *dev_path = network_device_get_nm_path (dev);
 
-		if (network_device_is_wireless (dev))
-		{
-			/* Build up the required args */
-			dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path,
-										DBUS_TYPE_STRING, &essid,
-										DBUS_TYPE_INVALID);
+		nm_info ("Forcing device '%s'", dev_path);
+
+		dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_INVALID);
+
+		if (essid)
+			dbus_message_append_args (message, DBUS_TYPE_STRING, &essid, DBUS_TYPE_INVALID);
+
+		if (opt)
+			success = wso_append_dbus_params (opt, essid, message);
 
-			/* If we have specific wireless security options, add them */
-			if (opt)
-				success = wso_append_dbus_params (opt, essid, message);
-		}
-		else
-		{
-			nm_info ("Forcing device '%s'\n", network_device_get_nm_path (dev));
-			dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_INVALID);
-		}
 		if (success)
 			dbus_connection_send (connection, message, NULL);
 		dbus_message_unref (message);
 	}
 	else
-		nm_warning ("Couldn't allocate the dbus message\n");
+		nm_warning ("Couldn't allocate the dbus message");
+}
+
+
+void nma_dbus_set_device_with_gconf_wso (DBusConnection *connection,
+								 NetworkDevice *dev,
+								 const char *essid,
+								 NMGConfWSO *opt)
+{
+	DBusMessage *	message;
+	gboolean		success = TRUE;
+
+	g_return_if_fail (connection != NULL);
+	g_return_if_fail (dev != NULL);
+	if (network_device_is_wireless (dev))
+		g_return_if_fail (essid != NULL);
+
+	if ((message = dbus_message_new_method_call (NM_DBUS_SERVICE, NM_DBUS_PATH, NM_DBUS_INTERFACE, "setActiveDevice")))
+	{
+		const char *dev_path = network_device_get_nm_path (dev);
+
+		nm_info ("Forcing device '%s'", dev_path);
+
+		dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_INVALID);
+
+		if (essid)
+			dbus_message_append_args (message, DBUS_TYPE_STRING, &essid, DBUS_TYPE_INVALID);
+
+		if (opt) {
+			DBusMessageIter iter;
+
+			dbus_message_iter_init_append (message, &iter);
+			success = nm_gconf_wso_serialize_dbus (opt, &iter);
+		}
+
+		if (success)
+			dbus_connection_send (connection, message, NULL);
+		dbus_message_unref (message);
+	}
+	else
+		nm_warning ("Couldn't allocate the dbus message");
 }
 
 
@@ -1315,18 +1376,18 @@ void nma_dbus_create_network (DBusConnection *connection, NetworkDevice *dev, co
 
 		if ((dev_path = network_device_get_nm_path (dev)))
 		{
-			nm_info ("Creating network '%s' on device '%s'.\n", essid, dev_path);
+			nm_info ("Creating network '%s' on device '%s'.", essid, dev_path);
 			dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path,
 										DBUS_TYPE_STRING, &essid,
 										DBUS_TYPE_INVALID);
 			wso_append_dbus_params (opt, essid, message);
 			dbus_connection_send (connection, message, NULL);
 		} else
-			nm_warning ("Could not get the device path!\n");
+			nm_warning ("Could not get the device path!");
 		dbus_message_unref (message);
 	}
 	else
-		nm_warning ("Couldn't allocate the dbus message\n");
+		nm_warning ("Couldn't allocate the dbus message");
 }
 
 
